/*  
 *    scope_graph.c
 *   ================
 *   Uwe Berger; 2013
 * 
 * Grafikroutinen fuer Scopeclock...
 * 
 * 
 * ---------
 * Have fun!
 *  
 */
 
#include <stdint.h>
#include "scope_graph.h"
#include "tux1.xbm"
//#include "tux2.xbm"
//#include "tux3.xbm"

#include "uartstdio.h"



// *********************************  
// http://de.wikipedia.org/wiki/Bresenham-Algorithmus
//	   
void plot_line(int x0, int y0, int x1, int y1)
{
	int dx =  my_abs(x1-x0), sx = x0<x1 ? 1 : -1;
	int dy = -my_abs(y1-y0), sy = y0<y1 ? 1 : -1;
	int err = dx+dy, e2; /* error value e_xy */
	for(;;){  /* loop */
		plot_point(x0,y0);
		if (x0==x1 && y0==y1) break;
		e2 = 2*err;
		if (e2 > dy) { err += dy; x0 += sx; } /* e_xy+e_x > 0 */
		if (e2 < dx) { err += dx; y0 += sy; } /* e_xy+e_y < 0 */
	}
}

// *********************************  	   
void plot_rectangle(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1)
{
	plot_line (x0, y0, x1, y0);
	plot_line (x1, y0, x1, y1);
	plot_line (x1, y1, x0, y1);
	plot_line (x0, y1, x0, y0);
}
 
// *********************************  	   
void plot_char (uint8_t px, uint8_t py, uint8_t c, uint8_t zoom) 
{
	uint8_t i, x, y, x0, y0;
	uint8_t first = 1;
	x0=y0=0;
	for (i=0; d[c][i][0]<E; i++) {
		if (d[c][i][0] == F) {
			first = 1;
			i++;
		}
		x=d[c][i][0] * zoom;
		y=(6-d[c][i][1]) * zoom;
		if (first==0) {
			plot_line(x0+px, y0+py, x+px, y+py);
		} else {
			first=0;	
		}
		x0=x;
		y0=y;
	}
} 


// *********************************  	   
void digital_clock (uint8_t x, uint8_t y, datetime_t now, uint8_t mark)
{	
	//x=30;
	//y=100;
	// Uhrzeit
	plot_char(x+(0*(DDX*ZOOM_T+DPX)), y,    now.hour/10, ZOOM_T);
	plot_char(x+(1*(DDX*ZOOM_T+DPX)), y,    now.hour%10, ZOOM_T);
	if (mark==EDIT_HOUR) {
		plot_rectangle(	x+(0*(DDX*ZOOM_T+DPX)), y,
						x+(2*(DDX*ZOOM_T+DPX)), y+(DDY*ZOOM_T));
	}
	plot_char(x+(2*(DDX*ZOOM_T+DPX)), y,         11, ZOOM_T);
	plot_char(x+(3*(DDX*ZOOM_T+DPX)), y,  now.minute/10, ZOOM_T);
	plot_char(x+(4*(DDX*ZOOM_T+DPX)), y,  now.minute%10, ZOOM_T);
	if (mark==EDIT_MINUTE) {
		plot_rectangle(	x+(3*(DDX*ZOOM_T+DPX)), y,
						x+(5*(DDX*ZOOM_T+DPX)), y+(DDY*ZOOM_T));
	}
	plot_char(x+(5*(DDX*ZOOM_T+DPX)), y,         11, ZOOM_T);
	plot_char(x+(6*(DDX*ZOOM_T+DPX)), y, now.secound/10, ZOOM_T);
	plot_char(x+(7*(DDX*ZOOM_T+DPX)), y, now.secound%10, ZOOM_T);
	if (mark==EDIT_SECOUND) {
		plot_rectangle(	x+(6*(DDX*ZOOM_T+DPX)), y,
						x+(8*(DDX*ZOOM_T+DPX)), y+(DDY*ZOOM_T));
	}
	// Datum
	x=x+(8*(DDX*ZOOM_T+DPX)-(10*(DDX*ZOOM_D+DPX)))/2;
	y=y-(DDY*ZOOM_T)-DPY;
	plot_char(x+(0*(DDX*ZOOM_D+DPX)), y,   now.day/10, ZOOM_D);
	plot_char(x+(1*(DDX*ZOOM_D+DPX)), y,   now.day%10, ZOOM_D);
	if (mark==EDIT_DAY) {
		plot_rectangle(	x+(0*(DDX*ZOOM_D+DPX)), y,
						x+(2*(DDX*ZOOM_D+DPX)), y+(DDY*ZOOM_D));
	}
	plot_char(x+(2*(DDX*ZOOM_D+DPX)), y,       10, ZOOM_D);
	plot_char(x+(3*(DDX*ZOOM_D+DPX)), y, now.month/10, ZOOM_D);
	plot_char(x+(4*(DDX*ZOOM_D+DPX)), y, now.month%10, ZOOM_D);
	if (mark==EDIT_MONTH) {
		plot_rectangle(	x+(3*(DDX*ZOOM_D+DPX)), y,
						x+(5*(DDX*ZOOM_D+DPX)), y+(DDY*ZOOM_D));
	}
	plot_char(x+(5*(DDX*ZOOM_D+DPX)), y,       10, ZOOM_D);
	plot_char(x+(6*(DDX*ZOOM_D+DPX)), y,        2, ZOOM_D); // 20xx
	plot_char(x+(7*(DDX*ZOOM_D+DPX)), y,        0, ZOOM_D);
	plot_char(x+(8*(DDX*ZOOM_D+DPX)), y,  now.year/10, ZOOM_D);
	plot_char(x+(9*(DDX*ZOOM_D+DPX)), y,  now.year%10, ZOOM_D);
	if (mark==EDIT_YEAR) {
		plot_rectangle(	x+(8*(DDX*ZOOM_D+DPX)), y,
						x+(10*(DDX*ZOOM_D+DPX)), y+(DDY*ZOOM_D));
	}
}	

// *********************************  	   
void draw_temperature(uint8_t x, uint8_t y, temperature_t t, uint8_t mode)
{
	uint32_t temp;
	if (mode==0)
		temp = t.celsius;
	else 
		temp = t.fahrenheit;
	plot_char(x+(0*(DDX*ZOOM_TEMP+DPX)), y, temp/100, ZOOM_TEMP);
	temp=temp%100;
	plot_char(x+(1*(DDX*ZOOM_TEMP+DPX)), y, temp/10, ZOOM_TEMP);
	plot_char(x+(2*(DDX*ZOOM_TEMP+DPX)), y, 10, ZOOM_TEMP);
	plot_char(x+(3*(DDX*ZOOM_TEMP+DPX)), y, temp%10, ZOOM_TEMP);
	plot_char(x+(4*(DDX*ZOOM_TEMP+DPX)), y, 12, ZOOM_TEMP);
	if (mode==TEMP_CELSIUS)
		plot_char(x+(5*(DDX*ZOOM_TEMP+DPX)), y, 15, ZOOM_TEMP);
	else
		plot_char(x+(5*(DDX*ZOOM_TEMP+DPX)), y, 16, ZOOM_TEMP);
}

// *********************************  	   
void draw_all_pixel(void)
{
	uint16_t i, j;
	for (i=0; i<=255; i++)
		for (j=0; j<=255; j++)
			plot_point(j, i);
}

// *********************************  	   
void analog_clock  (datetime_t now)
{	
	uint8_t x, y;
	int x0,x1,y0,y1,h,d;
	uint8_t i;
	// Ziffernblatt
	for (i=0; i<60; i=i+5) {
	y0 = MAX_Y-(R - R * my_cos(i)/1000);
	x0 = R + R * my_sin(i)/1000;
	y1 = MAX_Y-(R - (R-5) * my_cos(i)/1000);
	x1 = R + (R-5) * my_sin(i)/1000;
	plot_line(x0, y0, x1, y1);
	// mit Ziffern...?
		if (1) {
			y = R - (R-20) * my_cos(i)/1000;
			x = R + (R-20) * my_sin(i)/1000;
			d = i/5;
			if ((d>0) && (d<10)) {
				plot_char(x-DDX*3/2, MAX_Y-y-DDY*3/2, d, 3);
			} else {
				if (d==0) d=12;
				plot_char(x-DDX*3-1, MAX_Y-y-DDY*3/2, 1, 3);
				plot_char(x+DDX*3-1, MAX_Y-y-DDY*3/2, d%10, 3);
			}
		}
	}
	// Minutenpunkte
	for (i=0; i<60; i++) {
		y0 = MAX_Y-(R - R * my_cos(i)/1000);
		x0 = R + R * my_sin(i)/1000;
		plot_point(x0, y0);
	}
	// Sekunden-Zeiger (einfacher Strich vom Mittelpunkt...)
	y0 = MAX_Y - (R - (R-5) * my_cos(now.secound)/1000);
	x0 =  R + (R-5) * my_sin(now.secound)/1000;
	plot_line(R, MAX_Y-R, x0, y0);
	// Minuten-Zeiger
	fat_hand(now.minute, 10);
	// Stunden-Zeiger
	if (now.hour > 11) {h = now.hour - 12;} else {h = now.hour;} 
	h = h * 5 + now.minute/12;
	fat_hand(h, 25);
}

// *********************************  	   
void fat_hand (int val, int dist)
{
	int x0, x1, x2, x3, y0, y1, y2, y3, ir;
	ir = R-5;	
	// 0 Grad
	y0 = MAX_Y - (R - (R - dist) * my_cos(val)/1000);
	x0 = R + (R - dist) * my_sin(val)/1000;
	// 180 Grad
	val = val + 30;
	if (val > 59) val = val - 60;
	y2 = MAX_Y - (R - (R-ir) * my_cos(val)/1000);
	x2 = R + (R-ir) * my_sin(val)/1000;
	// 90 Grad
	val = val + 15;
	if (val > 59) val = val - 60;
	y3 = MAX_Y - (R - (R-ir) * my_cos(val)/1000);
	x3 = R + (R-ir) * my_sin(val)/1000;
	// 180 Grad
	val = val + 30;
	if (val > 59) val = val - 60;
	y1 = MAX_Y - (R - (R-ir) * my_cos(val)/1000);
	x1 = R + (R-ir) * my_sin(val)/1000;
	// ...und Zeiger zeichnen	
	plot_line(x0, y0, x1, y1);
	plot_line(x1, y1, x2, y2);
	plot_line(x2, y2, x3, y3);
	plot_line(x3, y3, x0, y0);
}


// *********************************  	 
void draw_xbm(uint8_t x, uint8_t y, uint8_t width, uint8_t height, unsigned char *xbm)
{
	uint8_t i, j, cols;
	uint8_t col_x=x;
	// Anzahl Bytes pro Zeile berechnen
	cols = width/8;
	if (width%8) cols++;
	// ueber jede Zeile...
	for (j=0; j<height; j++) { 
		// ueber jedes Byte einer Zeile...
		for (i=0; i<cols; i++) {
			// ist ueberhaupt ein Bit gesetzt?
			if (*xbm) {
				// ueber jedes Bit des Bytes... (for-Schleife ist langsamer!)
				if (*xbm & 1)   plot_point(col_x, y);	
				col_x++;
				if (*xbm & 2)   plot_point(col_x, y);	
				col_x++;
				if (*xbm & 4)   plot_point(col_x, y);	
				col_x++;
				if (*xbm & 8)   plot_point(col_x, y);	
				col_x++;
				if (*xbm & 16)  plot_point(col_x, y);	
				col_x++;
				if (*xbm & 32)  plot_point(col_x, y);	
				col_x++;
				if (*xbm & 64)  plot_point(col_x, y);	
				col_x++;
				if (*xbm & 128) plot_point(col_x, y);	
				col_x++;
			} else {
				col_x=col_x+8;
			}
			// naechstes Byte aus Bild-Array
			xbm++;
		}
		y--;		// naechste Zeile...
		col_x=x;	// ...beginnt wieder bei x
	}
}



// *********************************  	 
void draw_tux1(uint8_t x, uint8_t y) 
{
	draw_xbm(x, y, tux1_width, tux1_height, tux1_bits);	
//	draw_xbm(x, y, tux2_width, tux2_height, tux2_bits);	
//	draw_xbm(x, y, tux3_width, tux3_height, tux3_bits);	
}


// *********************************  	 
void plot_circle(int x0, int y0, int radius)
{
	int f = 1 - radius;
	int ddF_x = 0;
	int ddF_y = -2 * radius;
	int x = 0;
	int y = radius;
 
	plot_point(x0, y0 + radius);
	plot_point(x0 + radius, y0);
	plot_point(x0 - radius, y0);
	plot_point(x0, y0 - radius);
	while(x < y) {
		if(f >= 0) {
			y--;
			ddF_y += 2;
			f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x + 1;
 
		plot_point(x0 + x, y0 + y);
		plot_point(x0 - x, y0 + y);
		plot_point(x0 + x, y0 - y);
		plot_point(x0 - x, y0 - y);
		plot_point(x0 + y, y0 + x);
		plot_point(x0 - y, y0 + x);
		plot_point(x0 + y, y0 - x);
		plot_point(x0 - y, y0 - x);
	}
}


